// HotFontCombo.cpp : implementation file
//

#include "stdafx.h"
#include "HotFontCombo.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CHotFontCombo

CHotFontCombo::CHotFontCombo()
{
	m_clr3DFace = GetSysColor(COLOR_3DFACE);
	m_clr3DLight = GetSysColor(COLOR_3DLIGHT);
	m_clr3DHilight = GetSysColor(COLOR_3DHILIGHT);
	m_clr3DShadow = GetSysColor(COLOR_3DSHADOW);
	m_clr3DDkShadow = GetSysColor(COLOR_3DDKSHADOW);

	m_fGotFocus = false;
	m_fTimerSet = false;

	m_strSelectedFont = _T("");
}

CHotFontCombo::~CHotFontCombo()
{
}


BEGIN_MESSAGE_MAP(CHotFontCombo, CComboBox)
	//{{AFX_MSG_MAP(CHotFontCombo)
	ON_WM_PAINT()
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_WM_NCMOUSEMOVE()
	ON_WM_SYSCOLORCHANGE()
	ON_CONTROL_REFLECT(CBN_SETFOCUS, OnSetfocus)
	ON_CONTROL_REFLECT(CBN_KILLFOCUS, OnKillfocus)
	ON_WM_FONTCHANGE()
	ON_CONTROL_REFLECT(CBN_SELCHANGE, OnSelchange)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHotFontCombo message handlers

void CHotFontCombo::OnPaint() 
{
	Default();

	if (m_fGotFocus) {
		DrawBorder();
	} else {
		DrawBorder(false);
	}
}

void CHotFontCombo::OnSetFocus(CWnd* pOldWnd) 
{
	CComboBox::OnSetFocus(pOldWnd);
	
	m_fGotFocus = true;
	DrawBorder();
}

void CHotFontCombo::OnKillFocus(CWnd* pNewWnd) 
{
	CComboBox::OnKillFocus(pNewWnd);
	
	m_fGotFocus = false;
	DrawBorder(false);
}

void CHotFontCombo::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (!m_fGotFocus) {
		if (!m_fTimerSet) {
			DrawBorder();
			SetTimer(1, 10, NULL);
			m_fTimerSet = true;
		}
	}
	
	CComboBox::OnMouseMove(nFlags, point);
}

void CHotFontCombo::OnTimer(UINT nIDEvent) 
{
	POINT pt;
	GetCursorPos(&pt);
	CRect rcItem;
	GetWindowRect(&rcItem);

	if(!rcItem.PtInRect(pt)) {
		KillTimer(1);

		m_fTimerSet = false;

		if (!m_fGotFocus) {
			DrawBorder(false);
		}
		return;
	}
	
	CComboBox::OnTimer(nIDEvent);
}

void CHotFontCombo::OnNcMouseMove(UINT nHitTest, CPoint point) 
{
	if (!m_fGotFocus) {
		if (!m_fTimerSet) {
			DrawBorder();
			SetTimer(1, 10, NULL);
			m_fTimerSet = true;
		}
	}
	
	CComboBox::OnNcMouseMove(nHitTest, point);
}

void CHotFontCombo::OnSysColorChange() 
{
	CComboBox::OnSysColorChange();
	
	m_clr3DFace = GetSysColor(COLOR_3DFACE);
	m_clr3DLight = GetSysColor(COLOR_3DLIGHT);
	m_clr3DHilight = GetSysColor(COLOR_3DHILIGHT);
	m_clr3DShadow = GetSysColor(COLOR_3DSHADOW);
	m_clr3DDkShadow = GetSysColor(COLOR_3DDKSHADOW);
}

void CHotFontCombo::OnSetfocus() 
{
	m_fGotFocus = true;
	DrawBorder();
}

void CHotFontCombo::OnKillfocus() 
{
	m_fGotFocus = false;
	DrawBorder(false);
}

void CHotFontCombo::DrawBorder(bool fHot)
{
	CDC *pDC = GetDC();
	CRect rcItem;
	DWORD dwExStyle = GetExStyle();

	GetWindowRect(&rcItem);
	ScreenToClient(&rcItem);

	if (!IsWindowEnabled()) {
		fHot = true;
	}

	// make sure that rcItem start at the smallest point
	if (dwExStyle & (WS_EX_STATICEDGE | WS_EX_DLGMODALFRAME)) {
		if (dwExStyle & WS_EX_STATICEDGE) {
			rcItem.DeflateRect(1, 1, 0, 0);
		}
		if (dwExStyle & WS_EX_DLGMODALFRAME) {
			rcItem.DeflateRect(1, 1, 0, 0);
		}
		rcItem.DeflateRect(1, 1, 0, 0);
	} else {
		rcItem.DeflateRect(1, 1);
	}

	if (fHot) {
		pDC->Draw3dRect(rcItem, m_clr3DDkShadow, m_clr3DLight);
		rcItem.InflateRect(1, 1);
		pDC->Draw3dRect(rcItem, m_clr3DShadow, m_clr3DHilight);

		if (dwExStyle & WS_EX_DLGMODALFRAME) {
			pDC->Draw3dRect(rcItem, m_clr3DShadow, m_clr3DFace);
			rcItem.InflateRect(1, 1, 0, 0);
			if (dwExStyle & WS_EX_STATICEDGE) {
				rcItem.DeflateRect(0, 0, 2, 2);
			} else {
				rcItem.DeflateRect(0, 0, 1, 1);
			}
			pDC->Draw3dRect(rcItem, m_clr3DDkShadow, m_clr3DDkShadow);
		}

		if (dwExStyle & WS_EX_STATICEDGE) {
			if (dwExStyle & WS_EX_DLGMODALFRAME) {
				rcItem.InflateRect(1, 1);
			} else {
				rcItem.InflateRect(1, 1, 0, 0);
			}
			pDC->Draw3dRect(rcItem, m_clr3DShadow, m_clr3DHilight);
		}
	} else {
		pDC->Draw3dRect(rcItem, m_clr3DFace, m_clr3DFace);
		rcItem.InflateRect(1, 1);
		pDC->Draw3dRect(rcItem, m_clr3DFace, m_clr3DFace);

		if (dwExStyle & WS_EX_DLGMODALFRAME) {
			rcItem.InflateRect(1, 1, 0, 0);
			if (dwExStyle & WS_EX_STATICEDGE) {
				rcItem.InflateRect(0, 0, 1, 1);
				pDC->Draw3dRect(rcItem, m_clr3DFace, m_clr3DFace);
				rcItem.DeflateRect(0, 0, 3, 3);
			} else {
				rcItem.DeflateRect(0, 0, 1, 1);
			}
			pDC->Draw3dRect(rcItem, m_clr3DFace, m_clr3DFace);
		}

		if (dwExStyle & WS_EX_STATICEDGE) {
			if (dwExStyle & WS_EX_DLGMODALFRAME) {
				rcItem.InflateRect(1, 1);
			} else {
				rcItem.InflateRect(1, 1, 0, 0);
			}
			pDC->Draw3dRect(rcItem, m_clr3DFace, m_clr3DFace);
		}
	}

	ReleaseDC(pDC);
}

/////////////////////////////////////////////////////////////////////
// This is where the code for this article begins. All the above code
// is from my previous article "Hot Combo Box"
/////////////////////////////////////////////////////////////////////

void CHotFontCombo::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
	CDC dc;
	CBrush brBackground, *pOldBrush;
	CPen penOutline, *pOldPen;
	CFont fntFont, *pOldFont;

	// use an MFC CDC wrapper class - makes things a little easier
	dc.Attach(lpDrawItemStruct->hDC);

	// only draw valid items.
	if ((int)lpDrawItemStruct->itemID >= 0) {
		CString strText;

		// get the name of the font we'll be using (and the text to draw of course)
		GetLBText(lpDrawItemStruct->itemID, strText);

		// create a font with the name of the item to be drawn. Each item appears in it's own font
		fntFont.CreatePointFont(110, strText);
		pOldFont = dc.SelectObject(&fntFont);

		// set the colours depending on whether the item is highlighted or not
		if (lpDrawItemStruct->itemState & ODS_SELECTED) {
			brBackground.CreateSolidBrush(GetSysColor(COLOR_HIGHLIGHT));
			penOutline.CreatePen(PS_SOLID, 0, GetSysColor(COLOR_HIGHLIGHT));
			dc.SetTextColor(GetSysColor(COLOR_HIGHLIGHTTEXT));
		} else {
			brBackground.CreateSolidBrush(GetSysColor(COLOR_WINDOW));
			penOutline.CreatePen(PS_SOLID, 0, GetSysColor(COLOR_WINDOW));
			dc.SetTextColor(GetSysColor(COLOR_WINDOWTEXT));
		}

		// draw the background
		pOldBrush = dc.SelectObject(&brBrush);
		pOldPen = dc.SelectObject(&penOutline);
		dc.Rectangle(&lpDrawItemStruct->rcItem);

		// draw the text in the selected font.
		dc.SetBkMode(TRANSPARENT);
		dc.TextOut(lpDrawItemStruct->rcItem.left, lpDrawItemStruct->rcItem.top, strText);
		
		// select old resource back into device context to preserve system resources
		dc.SelectObject(pOldFont);
		dc.SelectObject(pOldBrush);
		dc.SelectObject(pOldPen);
	}

	// detach the DC again, thanks very much
	dc.Detach();
}

void CHotFontCombo::OnFontChange() 
{
	CComboBox::OnFontChange();

	// if a font is added to Windows or deleted from Windows the WM_FONTCHANGE message is sent to all
	// applications.
	InitialiseFontList();
}

int CALLBACK EnumFontFamExProc(
							   ENUMLOGFONTEX *lpelfe,    // logical-font data
							   NEWTEXTMETRICEX *lpntme,  // physical-font data
							   DWORD FontType,           // type of font
							   LPARAM lParam             // application-defined data
)
{
	CHotFontCombo *pwndCombo = (CHotFontCombo *)lParam;
	
	// simply add the font to the combo's list
	int nIndex = pwndCombo->AddString((char *)lpelfe->elfLogFont.lfFaceName);

	return 1;
}


void CHotFontCombo::InitialiseFontList()
{
	HDC hDC = ::GetDC(m_hWnd);
	LOGFONT lf;

	// empty the combo list in case this is in response to a WM_FONTCHANGE message
	ResetContent();

	// initialise the LOGFONT structure for the font enumeration
	lf.lfCharSet = ANSI_CHARSET;
	*lf.lfFaceName = 0;
	lf.lfPitchAndFamily = 0;
	EnumFontFamiliesEx(hDC, &lf, (FONTENUMPROC)EnumFontFamExProc, (LPARAM)this, 0);

	// make sure that the currently selected font appears
	int nIndex = FindStringExact(0, m_strSelectedFont);
	if (nIndex != CB_ERR) {
		SetCurSel(nIndex);
	}
}

int CHotFontCombo::CompareItem(LPCOMPAREITEMSTRUCT lpCompareItemStruct) 
{
	LPCTSTR lpszText1 = (LPCTSTR)lpCompareItemStruct->itemData1;
	LPCTSTR lpszText2 = (LPCTSTR)lpCompareItemStruct->itemData2;

	return strcmp(lpszText2, lpszText1);
}

void CHotFontCombo::OnSelchange() 
{
	int nIndex = GetCurSel();

	// make a record of the font selected
	if (nIndex != CB_ERR) {
		GetLBText(nIndex, m_strSelectedFont);
	}
}
